import { IAutoload } from 'najs-binding';
import { EloquentTimestamps, EloquentSoftDelete } from './EloquentMetadata';
import { IEloquentDriver } from '../drivers/interfaces/IEloquentDriver';
import { Collection } from 'collect.js';
/**
 * Base class of an Eloquent, handles proxy attributes, contains cross-driver features like
 *   - fill
 *   - touch
 *   - member Querying
 *   - static Querying
 */
export declare abstract class Eloquent<Record extends Object = {}> implements IAutoload {
    protected attributes: Record;
    protected driver: IEloquentDriver<Record>;
    protected temporarySettings?: {
        fillable?: string[];
        guarded?: string[];
        visible?: string[];
        hidden?: string[];
    };
    protected fillable?: string[];
    protected guarded?: string[];
    protected visible?: string[];
    protected hidden?: string[];
    protected timestamps?: EloquentTimestamps | boolean;
    protected softDeletes?: EloquentSoftDelete | boolean;
    protected table?: string;
    protected collection?: string;
    protected schema?: Object;
    protected options?: Object;
    constructor();
    constructor(data: Object);
    constructor(data: Record);
    static register(model: typeof Eloquent): void;
    abstract getClassName(): string;
    getModelName(): string;
    getAttribute(name: string): any;
    setAttribute(name: string, value: any): boolean;
    toObject(): Object;
    toJSON(): Object;
    toJson(): Object;
    fill(data: Object): this;
    forceFill(data: Object): this;
    getFillable(): string[];
    getGuarded(): string[];
    isFillable(key: string): boolean;
    isGuarded(key: string): boolean;
    getVisible(): string[];
    getHidden(): string[];
    isVisible(key: string): boolean;
    isHidden(key: string): boolean;
    protected isInWhiteList(key: string, whiteList: string[], blackList: string[]): boolean;
    protected isInBlackList(key: string, blackList: string[]): boolean;
    private mergeWithTemporarySetting(name, value);
    private concatTemporarySetting(name, value);
    markFillable(key: string): this;
    markFillable(keys: string[]): this;
    markFillable(...keys: Array<string>): this;
    markFillable(...keys: Array<string[]>): this;
    markFillable(...args: Array<string | string[]>): this;
    markGuarded(key: string): this;
    markGuarded(keys: string[]): this;
    markGuarded(...keys: Array<string>): this;
    markGuarded(...keys: Array<string[]>): this;
    markGuarded(...args: Array<string | string[]>): this;
    markVisible(key: string): this;
    markVisible(keys: string[]): this;
    markVisible(...keys: Array<string>): this;
    markVisible(...keys: Array<string[]>): this;
    markVisible(...args: Array<string | string[]>): this;
    markHidden(key: string): this;
    markHidden(keys: string[]): this;
    markHidden(...keys: Array<string>): this;
    markHidden(...keys: Array<string[]>): this;
    markHidden(...args: Array<string | string[]>): this;
    newInstance(data?: Object | Record): this;
    newCollection(dataset: Array<Object | Record>): Collection<this>;
    protected getReservedNames(): Array<string>;
}
